package ihm;

import moteur.MoteurIHM;
import moteur.ET;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;

public class IHM extends JFrame implements ActionListener, MouseListener, IHMMoteur
{
    // Composants servant  l'IHM Principale
    private Dimension screenSize;
    private JMenuItem nouveau, sauvegarder, sauvegarderSous, ouvrir, proprietes, preferences, quitter, aPropos ;
    // Boutons et icones du menu gnral
    private JButton bNouveau, bSauvegarder, bSauvegarderSous, bOuvrir, bQuitter, bProprietes ;
    private ImageIcon iNouveau, iSauvegarder, iOuvrir, iQuitter, iProprietes, iPreferences, iAPropos ;

    // Elments du menu prfrences
    private JDialog fPreferences ;
    private JButton bValider, bAnnuler ;
    private JCheckBox diagramme, fenetre, son, logs ;
    private boolean boolLogs, boolDiagramme, boolSon, boolFenetre ;
	 private String pathPref = "app.dat"; 

    private static File fichierCourant, fichierPreferences;
    
   /**
    * Actions de la souris.
    */
   private static final int MA_SELECT   = 0;
   private static final int MA_ADD      = 1;

   private static final String[] imageURLs = {
      "icones/other.png",
      "icones/pc.png",
      "icones/router.png",
      "icones/printer.png",
      "icones/switch.png",
      "icones/server.png",
      "icones/hub.png"
   };

   private static final int DEFAULT_SYMB_WIDTH  = 48;
   private static final int DEFAULT_SYMB_HEIGHT = 48;

/**
 * Le panneau des outils de cration
 */
    private PanelCreation panelCreation; 
    
/**
 * Le panneau de dessin du graphe
 */
    private PanelDraw panelDraw; 
    
/**
 * Le panneau des conseils / informations
 */
    private PanelHint panelHint; 

/**
 * Maintient une liste des images utilises pour les
 * symboles.
 */
   private Image[] imagesSymb;
   
/**
 * L'icone par dfaut gnr dynamiquement.
 * Permet d'avoir quelquechose  afficher mme
 * si aucun fichier n'est disponible
 */
   private Image defaultIcon = null;
   
/**
 * Action  effectuer avec la souris
 */
   private int mouseAction = MA_SELECT;
   
/**
 * Le type de symbole  ajouter
 */
   private int addSymbType = 0;

/**
 * Le symbole slctionn. null si aucun
 */
   private Symbole selSymb = null;

/**
 * L'interface pour utiliser le Moteur
 */
   private MoteurIHM mi = null;

/**
 * La liste des symboles prsents dans le schma du rseau
 */
    public Vector symboles;
    
/**
 * La pile d'alertes, ausis utilise par le thread d'alertes.
 * Tous els accs  cette liste doivent tre synchroniss dessus.
 */
   private Vector alerts;

/**
 * La liste des rseaux.
 */
   private Vector nets;
   
/**
 * Thread de traitement des alertes.
 */
   private AlertThread alertThread = null;
   
   public IHM()
   {
      symboles = new Vector ();
      alerts = new Vector();
      nets = new Vector();

      // Permet d' avoir une application directement plein cran
      screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      setSize(screenSize);
      setTitle("The Yellow SNMParine v. 0.00002 beta night build!");
      //setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
      
      JPanel pFenetre = new JPanel(new BorderLayout()) ;
      
      initIcones();
      setJMenuBar (menu());
      pFenetre.add (boutons(), BorderLayout.NORTH);
      pFenetre.add (panelAffichage(), BorderLayout.CENTER);
      
      setContentPane(pFenetre);
      
      addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
        exit(true);
        }
      });
      
      alertThread = new AlertThread (alerts, symboles, panelDraw);
   }

   private void exit(boolean promptConfirm) {
      boolean canExit;
      if (promptConfirm)
         canExit = (JOptionPane.showConfirmDialog(null, "Etes-vous sr de vouloir quitter?", "Quitter?", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
      else
         canExit = true;
      if (canExit) {
         mi.endApp();
      }
   } // end exit

    private void initPreferences()
    {
        fichierPreferences = new File (pathPref);
		if(!(fichierPreferences.exists()))
       	{
			boolLogs = true;
			boolDiagramme = true;
			boolSon = true;
			boolFenetre = true;
			mi.logOn();
		}
		else
		{
			try
			{
				BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fichierPreferences)));
				conversionPreferences (br.readLine());
				br.close();
				if(boolLogs)
					mi.logOn();
				else
					mi.logOff();
			}
			catch (IOException ioe)
			{ 
				boolLogs = true;
				boolDiagramme = true;
				boolSon = true;
				boolFenetre = true;
				mi.logOn();	
				System.out.println("Problme de lecture du fichier de prfrences");
			}
		}

    }

    private JPanel panelAffichage()
    {   
      // Cre le panneau pour les outils de cration
      // On instancie la classe que l'on a contruite  cet effet: PanelCreation
      panelCreation = new PanelCreation (this);
      
      // Cre le panneau pour la zone de dessin
      // On instancie la classe que l'on a construite  cet effet: PanelDraw
      panelDraw = new PanelDraw (this);

      // Cre le panneau pour la zone de messages d'information
      // On instancie la classe que l'on a contruite  cet effet: PanelHint
      panelHint = new PanelHint(this);
      
      return new GraphPanel (panelCreation, panelDraw, panelHint);
    }
    
    private void initIcones()
    {
        // Initialisations des icones du menu gnral
        iNouveau = new ImageIcon(IHM.class.getResource("icones/nouveau.png"));
        iSauvegarder= new ImageIcon(IHM.class.getResource("icones/sauvegarder.png"));
        iOuvrir = new ImageIcon(IHM.class.getResource("icones/ouvrir.png"));
        iQuitter = new ImageIcon(IHM.class.getResource("icones/quitter.png"));
        iProprietes= new ImageIcon(IHM.class.getResource("icones/proprietes.png"));
        iPreferences= new ImageIcon(IHM.class.getResource("icones/preferences.png"));
        iAPropos = new ImageIcon(IHM.class.getResource("icones/aPropos.png"));

    }
/**
 * Cre l'icne par dfaut
 */
   private Image makeDefaultIcon() {
      BufferedImage img = new BufferedImage (DEFAULT_SYMB_WIDTH, DEFAULT_SYMB_WIDTH, BufferedImage.TYPE_4BYTE_ABGR);
      Graphics2D g = img.createGraphics();
      g.setColor (Color.yellow);
      g.fillRoundRect (0, 0, DEFAULT_SYMB_WIDTH-1, DEFAULT_SYMB_HEIGHT-1, DEFAULT_SYMB_WIDTH/6, DEFAULT_SYMB_HEIGHT/6);
      g.setColor (Color.black);
      g.drawRoundRect (0, 0, DEFAULT_SYMB_WIDTH-1, DEFAULT_SYMB_HEIGHT-1, DEFAULT_SYMB_WIDTH/6, DEFAULT_SYMB_HEIGHT/6);
      g.setFont (new Font("Default", Font.BOLD, DEFAULT_SYMB_HEIGHT-2));
      g.setColor (Color.gray);
      g.drawString ("?", DEFAULT_SYMB_WIDTH/6, DEFAULT_SYMB_HEIGHT-DEFAULT_SYMB_HEIGHT/6);
      return img;
   }
/**
 * Retourne l'icne par dfaut aprs l'avoir cre si besoin est.
 */
   private Image getDefaultIcon() {
      if (defaultIcon == null) {
         defaultIcon = makeDefaultIcon();
      }
      return defaultIcon;
   }
   
    private JMenuBar menu()
    {
        JMenuBar jmb = new JMenuBar();

        JMenu fichier = new JMenu("Fichier");
        nouveau = new JMenuItem("Nouveau", iNouveau);
        nouveau.addActionListener(this);
        nouveau.setAccelerator(KeyStroke.getKeyStroke('N', Event.CTRL_MASK));
        sauvegarder = new JMenuItem("Sauvegarder", iSauvegarder);
        sauvegarder.addActionListener(this);
        sauvegarder.setAccelerator(KeyStroke.getKeyStroke('S', Event.CTRL_MASK));
        sauvegarderSous = new JMenuItem("Sauvegarder Sous", iSauvegarder);
        sauvegarderSous.addActionListener(this);
        sauvegarderSous.setAccelerator(KeyStroke.getKeyStroke('S', Event.ALT_MASK));
        ouvrir = new JMenuItem("Ouvrir", iOuvrir);
        ouvrir.addActionListener(this);
        ouvrir.setAccelerator(KeyStroke.getKeyStroke('O', Event.CTRL_MASK));
        quitter = new JMenuItem("Quitter", iQuitter);
        quitter.addActionListener(this);
        quitter.setAccelerator(KeyStroke.getKeyStroke('X', Event.CTRL_MASK));
        fichier.add(nouveau);
        fichier.add(sauvegarder);
        fichier.add(sauvegarderSous);
        fichier.add(ouvrir);
        fichier.add(quitter);

        JMenu affichage = new JMenu("Affichage");
        proprietes = new JMenuItem("Proprits de l'lment slectionn", iProprietes);
        proprietes.addActionListener(this);
        proprietes.setAccelerator(KeyStroke.getKeyStroke('P', Event.CTRL_MASK));
        affichage.add(proprietes);

        JMenu outils = new JMenu("Outils");
        preferences = new JMenuItem("Prfrences", iPreferences);
        preferences.addActionListener(this);
        preferences.setAccelerator(KeyStroke.getKeyStroke('P', Event.CTRL_MASK));
        outils.add(preferences);

        JMenu aide = new JMenu("?");
        aPropos = new JMenuItem("A Propos de The Yellow SNMParine !", iAPropos);
        aPropos.addActionListener(this);
        aPropos.setAccelerator(KeyStroke.getKeyStroke('H', Event.CTRL_MASK));
        aide.add(aPropos);

        jmb.add(fichier);
        jmb.add(affichage);
        jmb.add(outils);
        jmb.add(aide);

        return (jmb);
    }

    private JPanel boutons()
    {
        JPanel pBouton = new JPanel();
        pBouton.setLayout(new FlowLayout());
        pBouton.setBackground(Color.yellow);

        bNouveau = new JButton("", iNouveau);
        bNouveau.addActionListener(this);
        pBouton.add(bNouveau);
        bSauvegarder = new JButton("", iSauvegarder);
        bSauvegarder.addActionListener(this);
        pBouton.add(bSauvegarder);
        bOuvrir = new JButton("", iOuvrir);
        bOuvrir.addActionListener(this);
        pBouton.add(bOuvrir);
        bProprietes = new JButton("", iProprietes);
        bProprietes.addActionListener(this);
        pBouton.add(bProprietes); 
        bQuitter = new JButton("", iQuitter);
        bQuitter.addActionListener(this);
        pBouton.add(bQuitter);

        return pBouton;
    }
    
    private File selectionFichier()
    {
        JFileChooser file = new JFileChooser();
        JFrame jf = new JFrame();
        file.showOpenDialog(jf);
        jf.setVisible(true);
        fichierCourant = file.getSelectedFile() ;
        jf.setVisible(false);

        if (fichierCourant == null || fichierCourant.exists())
        {
            setHint("Fichier Source : " + fichierCourant);
            return fichierCourant ;
        }
        else
        {
            setHint("Le fichier saisi n'existe pas !\n");
            return null;
        }
    }

    private File enregistrerSous()
    {
        JFileChooser fileSave = new JFileChooser() ;
        fileSave.showSaveDialog (new JFrame()) ;
         String ext1 = ".sav";
         if(fileSave.getSelectedFile() == null) 
            return null;
         String fichier = (fileSave.getSelectedFile()).getAbsolutePath();        
         if(! fichier.endsWith(ext1) )
            fichier = fichier + ext1;

        File f = new File(fichier);

        return f ;
    }

    private JDialog menuPreferences()
    {
        // la fenetre sera modale (true)
        fPreferences = new JDialog(this, "Prfrences", true);
        fPreferences.setDefaultCloseOperation(HIDE_ON_CLOSE);

        JPanel pCheckBox = new JPanel(new GridLayout(5, 1));

        pCheckBox.add(new JLabel("De quelle faon souhaitez-vous tre inform des alertes ?"));

        logs = new JCheckBox("Gestion de fichiers logs", boolLogs);
        diagramme = new JCheckBox("Modification de l'affichage du diagramme", boolDiagramme);
        son = new JCheckBox("Alerte sonore", boolSon);
        fenetre = new JCheckBox("Nouvelle fentre d'alerte", boolFenetre);
        pCheckBox.add(logs);
        pCheckBox.add(diagramme);
        pCheckBox.add(son);
        pCheckBox.add(fenetre);

        fPreferences.getContentPane().add(pCheckBox, "Center");
        fPreferences.getContentPane().add(pButtonValider(), "South");

        fPreferences.pack();
        fPreferences.setLocation((int)(screenSize.getWidth()/2-fPreferences.getWidth()/2), (int)(screenSize.getHeight()/2-fPreferences.getHeight()/2));

        fPreferences.setVisible(true);

        return fPreferences ;
    }

    private JPanel pButtonValider()
    {
        JPanel pButton = new JPanel(new FlowLayout());

        bValider = new JButton("Valider");
        bValider.addActionListener(this);
        pButton.add(bValider);

        bAnnuler = new JButton("Annuler");
        bAnnuler.addActionListener(this);
        pButton.add(bAnnuler);

        return pButton ;
    }

    private boolean writePreferences(boolean bLogs, boolean bDiagramme, boolean bSon, boolean bFenetre)throws IOException
    {
        
		//if (fichierPreferences == null)
			//fichierPreferences = new File (fichierCourant.getAbsolutePath().substring(0,fichierCourant.getName().length()-3)+"dat");
		fichierPreferences = new File(pathPref);
        PrintWriter pw = new PrintWriter(new OutputStreamWriter(new FileOutputStream(fichierPreferences)));

        pw.write(Boolean.toString(bLogs)+";"+Boolean.toString(bDiagramme)+";"+Boolean.toString(bSon)+";"+Boolean.toString(bFenetre));
        pw.close();
	
        return true ;
    }

    private void conversionPreferences(String s)
    {
        StringTokenizer st = new StringTokenizer(s, ";");
        if (st.countTokens() >= 4) {
           boolLogs = st.nextToken().equals("true");
           boolDiagramme = st.nextToken().equals("true");
           boolSon = st.nextToken().equals("true");
           boolFenetre = st.nextToken().equals("true");
        } else {
           boolLogs = false;
           boolDiagramme = false;
           boolSon = false;
           boolFenetre = false;
        }
    }
    
   private Network findNet (String network) {
      for (int i=0; i<nets.size(); i++) {
         if (network.equals (((Network)nets.elementAt(i)).getAddr())) {
            return (Network)nets.elementAt(i);
         }
      }
      Network n = new Network (network);
      nets.addElement(n);
      return n;
   }
   
   private void removeFromNet (Symbole s) {
      for (int i=0; i<nets.size(); i++) {
         ((Network)nets.elementAt(i)).removeSymb (s);
      }
   }

   private void addSymbInAll (Symbole s) {
      symboles.addElement (s);
      panelCreation.addSymbole(s);
   }
    
    private void newGraph() {
      panelCreation.removeAllSymbs();
      synchronized (alerts) {
         alerts.clear();
      }
      synchronized (symboles) {
         setSelection (null);
         symboles.clear();
      }
      synchronized (nets) {
         nets.clear();
      }
      synchronized (panelDraw) {
         panelDraw.repaint();
      }
      creatChangeTool (0);
      setHint ("Prt");
      setPos (0, 0);
   }
   
   private void openGraph (SymbBounds[] sb) {
      newGraph();
      int x = 0;
      int haut = 0;
      int y = haut;
      
      for (int i=0; i<sb.length; i++) {
         int id = sb[i].getId();
         Symbole s;
         if (sb[i].hasCoords()) {
            s = new Symbole (this, id, mi.getType(id), mi.isOnline(id), sb[i].getX(), sb[i].getY(), sb[i].getW(), sb[i].getH());
         } else {
            s = new Symbole (this, id, mi.getType(id), mi.isOnline(id), x, y);
            if (s != null) {
               x += s.getBounds().getWidth() + 5;
               if (x > getWidth()) {
                  x = 0;
                  y += s.getBounds().getHeight() + 5;
                  if (y > getHeight()) {
                     haut += 10;
                     y = haut;
                     if (haut > getHeight()) {
                        haut = 0;
                        // L, on aura tout fait pour que tout soit affich dans la fentre...
                     }
                  }
               }
            }
         }
         if (s != null) {
            s.setIp(mi.getIpContact (id));
            s.setName(mi.getName (id));
            if (s.getOnline()) {
               s.setNet (mi.getNetwork (id));
               findNet (s.getNet()).addSymb (s);
            }
            addSymbInAll(s);
         }
      }
      panelDraw.repaint();
   }

    public void actionPerformed (ActionEvent e)
    {
        if (e.getSource() == aPropos)
            JOptionPane.showMessageDialog(new JFrame()," The Yellow SNMParine ! v 0.00002 \n\n Auteurs : \n Leo Cacheux\n Yacine Hessas\n Gwenal Monthiers\n Adrien Morel\n Mohamed Ouldbatty\n Alan Tranouez\n\nProjet - Universit de Rouen\nAnne Universitaire 2005 - 2006");

        if (e.getSource() == quitter || e.getSource() == bQuitter)
            exit(true);

        if (e.getSource() == ouvrir || e.getSource() == bOuvrir)
        {
            File f = selectionFichier() ;
            
            if ( f != null )
            {
                SymbBounds[] result  = mi.open(f);
                if (result != null)
                {
                    fichierCourant = (f.getName().endsWith(".sav") ? f : null);
                    //fichierPreferences = new File (fichierCourant.getName().substring(0,fichierCourant.getName().length()-3)+"dat");
                    initPreferences();
                    openGraph (result);
                }
                else
                    JOptionPane.showMessageDialog(new JFrame(),"impossible d'ouvrir le fichier selectionn");
            }
        }

        if ((e.getSource() == sauvegarder) || (e.getSource() == bSauvegarder) || (e.getSource() == sauvegarderSous)) {
            if (symboles.size() > 0) {
               String message ;
               if ((e.getSource() == sauvegarderSous) || (fichierCourant == null))
               {
                  File f = enregistrerSous();
                  if (f != null) {
                     fichierCourant = f;
                  } else {
                     return;
                  }
               }
                SymbBounds [] tab = new SymbBounds[symboles.size()];
                for (int i=0; i<tab.length; i++) {
                  tab[i] = new SymbBounds ((Symbole)symboles.elementAt(i));
                }
                if (!((message = mi.save(fichierCourant,tab)).equals("true")))
                    JOptionPane.showMessageDialog(new JFrame(), message);
                else
                {
                    try
                    {
                        writePreferences(boolLogs, boolDiagramme, boolSon, boolFenetre);
                    }
                    catch(IOException ioe){System.out.println("Problme pour enregistrer les prfrences !") ;}
                }
            }
         }

        if (e.getSource() == nouveau || e.getSource() == bNouveau)
        {
            mi.delete();
            fichierCourant = null ;
            fichierPreferences = new File(pathPref);
            initPreferences();
            newGraph();
        }

        if (e.getSource() == proprietes || e.getSource() == bProprietes)
        {
            try
            {
               changeSelSymbProp ();
            }
            catch(Exception exc) {System.out.println("erreur");}
       }


        if (e.getSource() == preferences)
            menuPreferences();

        if (e.getSource() == bValider)
        {
            boolLogs = logs.isSelected();
            if (boolLogs)
            {
                mi.logOn();
            }
            else
            {
                mi.logOff();
            }

            boolDiagramme = diagramme.isSelected();

            boolSon = son.isSelected();

            boolFenetre = fenetre.isSelected();

            if (fichierPreferences != null)
            {
                try
                {
                    writePreferences(boolLogs, boolDiagramme, boolSon, boolFenetre);
                }
                catch(IOException ioe){System.out.println("Problme pour enregistrer les prfrences !") ;}
            }

            fPreferences.removeAll();
            fPreferences.setVisible(false);
        }

        if (e.getSource() == bAnnuler)
        {
            fPreferences.removeAll();
            fPreferences.setVisible(false);
        }
    }

    public void mousePressed(java.awt.event.MouseEvent e)
    {
      System.out.println ("Maintenant");
    }
    
    public void mouseClicked(java.awt.event.MouseEvent e) {}
    public void mouseReleased(java.awt.event.MouseEvent e) {}
    public void mouseEntered(java.awt.event.MouseEvent e) {}
    public void mouseExited(java.awt.event.MouseEvent e) {}
    
    
/**
 * Renvoie l'image correspondant au type de symbole donn.
 * Renvoie l'image par dfaut si le type n'est pas connu ou n'a pas d'image associe.
 */
   public Image getImage (int symbType) {
      if ((symbType >= 0) && (symbType < imagesSymb.length)) {
         return imagesSymb[symbType];
      } else {
         return getDefaultIcon();
      }
   }
   
/**
 * Renvoie un tableau des symboles se trouvant sous le point indiqu.
 * Le tableau contient les symboles dont un pixel est en (x,y).
 * L'ordre dans le tableau est: du plus rcemment ajout au plus ancien.
 */
   public Symbole[] symbolesAtPos (int x, int y) {
      Vector symbs = new Vector();
      for (int i=symboles.size()-1; (i >= 0); i--) {
         Symbole s = (Symbole)symboles.elementAt(i);
         if ( s.intersects (x, y, 1, 1) ) {
            symbs.addElement (s);
         }
      }
      Symbole[] ss = {};
      return (Symbole[])symbs.toArray(ss);
   }

/**
 * Renvoie le symbole actuellement slctionn, null si aucun.
 */ 
   public Symbole getSelection () {
      return selSymb;
   }

/**
 * Dfinit le symbole slctionn.
 * null anulle toute slction.
 */ 
   public void setSelection (Symbole sel) {
      selSymb = sel;
      panelCreation.setSelection(sel);
      if (sel != null) {
         setHint (sel.getName() + " slctionn");
      } else {
         setHint ("Aucune slction");
      }
      synchronized (panelDraw) {
         panelDraw.repaint();
      }
   }

/**
 * Renvoie le nombre de symboles du graphe du schma.
 */
   public int getSymbolesCount () {
      return symboles.size();
   }
   
/**
 * Renvoie le symbole  l'index donn de la liste.
 */
   public Symbole getSymbole (int index) {
      return (Symbole)symboles.elementAt(index);
   }

/**
 * Renvoie le nombre de rseaux du graphe du schma.
 */
   public int getNetsCount () {
      return nets.size();
   }
   
/**
 * Renvoie le rseau  l'index donn de la liste.
 */
   public Network getNet (int index) {
      return (Network)nets.elementAt(index);
   }   

/**
 * Ajoute un symbole dans le graphe  la position donne.
 */
   public Symbole addSymbole (int addX, int addY) {
      int elID = mi.addElementReseau (addSymbType);
      if (elID >= 0) {
         System.out.println ("Nouveau symbole en " + addX + "," + addY + ".");
         Symbole s = new Symbole (this, elID, addSymbType, false, addX, addY, 48, 48);
         if (s != null) {
            setHint ("Element " + s.getName() + " ajout");
            addSymbInAll (s);
            setSelection (s);
         }
         return s;
      } else {
         System.out.println ("IMPOSSIBLE D'AJOUTER L'ELEMENT. PAS COOL CA!");
         return null;
      }
   }
/**
 * Ajoute un symbole  la position 0, 0 (haut gauche).
 */
   public Symbole addSymbole () {
      return addSymbole(0, 0);
   }

/**
 * Supprime le symbole actuellement slctionn.
 * C'est la seule mthode qui permette de supprimer, il faut donc 
 * slctionner avant de supprimer.
 */
   public void removeSelected () {
      if (selSymb != null) {
         mi.removeElementReseau(selSymb.getID());      
         symboles.remove (selSymb);
         removeFromNet (selSymb);
         panelCreation.removeSymbole(selSymb);
         synchronized (panelDraw) {
            panelDraw.repaint();
         }
      }
   }

/**
 * Renvoie le nombre d'outils disponibles.
 */
   public int getToolsCount () {
      return ET.getTypesCount() + 1;
   }
   
/**
 * Renvoie le nom d'un outil ou une chaine vide si l'index est hors-limites.
 * /!\ L'algorithme de dtermination de l'outil utilis ici doit
 * /!\ tre le mme que celui des autres mthodes utilisant un
 * /!\ index d'outil en paramtre.
 */
   public String getToolName (int toolIndex) {
      if ((toolIndex >= 0) && (toolIndex < getToolsCount())) {
         if (toolIndex == 0) {
            return "Slcteur";
         } else {
            return ET.getTypeName (toolIndex-1);
         }
      } else {
         return "";
      }
   }
   
/**
 * Renvoie le nom de l'outil actuellement actif.
 * /!\ L'algorithme de dtermination de l'outil utilis ici doit
 * /!\ tre le mme que celui des autres mthodes utilisant un
 * /!\ index d'outil en paramtre.
 */
   public String getCurToolName () {
      switch (mouseAction) {
         case MA_SELECT:
            return getToolName (0);
         case MA_ADD: 
            return getToolName (addSymbType + 1);
         default:
            return "";
      }
   }
   
/**
 * Dfinit le conseil  afficher.
 */
   public void setHint (String s) {
      panelHint.setHint (s);
   }

/**
 * Dfinit la position  afficher.
 */ 
   public void setPos (int x, int y) {
      panelHint.setPos (x, y);
   }    

/**
 * Affiche la fenetre des proprits de l'lment slctionn,
 * et traite les valeurs de retour;
 */
   public void changeSelSymbProp () {
      if (selSymb != null) {
         if (!PropDialog.popPropDlg (this, "Proprits de: " + selSymb, selSymb.getID(), mi)) {
            System.out.println ("Impossible d'ouvrir les proprits");
         }
         int st = mi.getType (selSymb.getID());
         String si = mi.getIpContact (selSymb.getID());
         String sn = mi.getName (selSymb.getID());
         boolean so = mi.getOnline (selSymb.getID());
         
         if ((st >= 0) && (si != null) && (sn != null)) {
            selSymb.setType (st);
            selSymb.setIp (si);
            selSymb.setName (sn);
            selSymb.setOnline (so);
            removeFromNet (selSymb);
            if (so) {
               selSymb.setNet (mi.getNetwork (selSymb.getID()));
               System.out.println("Symbole online, network: " + selSymb.getNet());
               findNet (selSymb.getNet()).addSymb (selSymb);
            }
         }
         
         synchronized (panelDraw) {
            panelDraw.repaint();
         }
      }
   }
   
   public void showSelectedAlerts() {
      if (selSymb != null) {
         new AlertDialog (selSymb.popAllAlerts(), this, mi, "Alertes de " + selSymb.getName() + " (" + selSymb.getIp() + ")");
      }
      synchronized (panelDraw) {
         panelDraw.repaint();
      }      
   }
   
/****************************************************************************/
/****************************************************************************/
/********************* IMPLEMENTATION DE IHMMOTEUR **************************/
/****************************************************************************/
/****************************************************************************/
   public boolean init(MoteurIHM mi) {
      this.mi = mi;
      
      try {
         initPreferences();
      } catch (Exception e) {
         System.out.println ("Erreur d'initialisation de la fentre de prfrences.");
         return false;
      }

      fichierCourant = null;
      fichierPreferences = new File(pathPref);
   
      alertThread.start();
   
      // Chargement des icones
      imagesSymb = new Image[imageURLs.length];
      for (int i = 0; i < imageURLs.length; i++) {
         java.net.URL imageURL = IHM.class.getResource(imageURLs[i]);
         if (imageURL != null) {
            try {
               imagesSymb[i] = ImageIO.read(imageURL);
            } catch (Exception e) {
               // En cas d'erreur de chargement, on associe l'image par dfaut.
               imagesSymb[i] = getDefaultIcon();
               System.out.println ("Erreur de chargement de l'icne "+imageURLs[i]+". Utilisation de l'icne par dfaut.");
            }
         } else {
            // Si le fichier n'existe pas, on associe l'image par dfaut.
            imagesSymb[i] = getDefaultIcon();
            System.out.println ("Impossible de charger l'icne "+imageURLs[i]+". Utilisation de l'icne par dfaut.");
         }
      }
      
      //setSize(Toolkit.getDefaultToolkit().getScreenSize());
      setExtendedState (Frame.MAXIMIZED_BOTH);
      // Operations finales sur le "JFrame" pour le rendre affichable
      //pack();
      setLocationRelativeTo(null);
      setVisible(true);
      
      return true;
   }  // end init()

   public void finalize() {
      alertThread.finish();
      try { alertThread.join(5000); } catch (Exception e) {}
   } // end finalize()
   
   public void notifyAlert (int origID, String msg, Date date) {
      synchronized (alerts) {
         alerts.addElement (new Alert (origID, msg, date));
      }
   } // end notifyAlert()
       
/****************************************************************************/
/****************************************************************************/
/************************** Evnements du graphe ****************************/
/****************************************************************************/
/****************************************************************************/
   public void graphLeftClick (int cX, int cY) {
      switch (mouseAction) {
         case MA_SELECT:
            Symbole[] symbs = symbolesAtPos (cX, cY);
            if (symbs.length > 0) {
               setSelection (symbs[0]);
               //notifyAlert (symbs[0].getID(), "Mchante alerte", new Date());
            } else {
               setSelection (null);
            }
            break;
         case MA_ADD:
            addSymbole (cX, cY);
            break;
      }
   } // end graphLeftClick
   
   public void graphRightClick (int cX, int cY) {
      Symbole[] symbs = symbolesAtPos (cX, cY);
      if (symbs.length > 0) {
         creatChangeTool (0);
         setSelection (symbs[0]);
         new SymbMenu (this, symbs[0], panelDraw, cX+1, cY+1);
      }
   } // end graphRightClick

   public void graphMouseMoved (int mOldX, int mOldY, int mNewX, int mNewY, boolean dragging) {
      if (dragging) {
         if (selSymb != null) {
            selSymb.moveOf (mNewX-mOldX, mNewY-mOldY);
            setHint ("Dplacement de " + selSymb.getName());
            synchronized (panelDraw) {
               panelDraw.repaint();
            }
         }
      } else {
      }
      setPos (mNewX, mNewY);
   }
   
/****************************************************************************/
/****************************************************************************/
/************************ Evnements de cration ****************************/
/****************************************************************************/
/****************************************************************************/

/**
 * Dfinit l'outil slctionn.
 * /!\ L'algorithme de dtermination de l'outil utilis ici doit
 * /!\ tre le mme que celui des autres mthodes utilisant un
 * /!\ index d'outil en paramtre.
 */
   public void creatChangeTool (int toolIndex) {
      if ((toolIndex <= 0) || (toolIndex >= getToolsCount())) {
         mouseAction = MA_SELECT;
         panelCreation.setToolLabel (toolIndex);
      } else {
         mouseAction = MA_ADD;
         addSymbType = toolIndex - 1;
         panelCreation.setToolLabel (toolIndex);
      }
   }

} // end Interface