package ihm;

import java.awt.*;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.lang.Object;
import java.util.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.border.*;
import moteur.ET;

/**
 * C'est la classe qui dfinit la structure, le contenu, et le comportement
 * du panneau de cration. Pour l'ajouter  l'interface, on cre une
 * instance de cette classe et on l'ajoute au container principal de
 * l'application swing
 */
public class PanelCreation extends JPanel {

/**
 * Le label d'affichage des conseils.
 */
   private JLabel toolLabel; 
/**
 * La boite de slction des symboles.
 */
   private JComboBox cbxSymbs;
/**
 * Le bouton de suppression du symbole slctionn.
 */
   private JButton btSuppr; 

/**
 * La liste des boutons de crations.
 */
   private Vector buttons; 

/**
 * La rfrence  l'interface mre.
 */
   private IHM interf;


/**
 * Construit le "panel" avec un GridLayout ayant
 * le nombre de lignes et de colonnes spcifies
 */
   PanelCreation(IHM interf, int rows, int cols) {
      int i;
      
      this.interf = interf;
      
      // Creation d'une belle bordure compose pour ce panneau
      Border etched = BorderFactory.createEtchedBorder ();
      Border blueline = BorderFactory.createLineBorder (Color.blue);
      setBorder ((Border)BorderFactory.createCompoundBorder (etched, blueline));
      
      
      /* btPane: PANNEAU DES BOUTONS DE CREATION                               */
      
      
      // Creation d'un sous-panneau pour l'affichage des boutons
      JPanel btPane = new JPanel ();
      
      // Force l'affichage des boutons en grille avec les parametres spcifis
      btPane.setLayout (new GridLayout(rows, cols));
      
      btPane.setBorder (BorderFactory.createEtchedBorder ());
      
      JSpacedButton button;
      // Ajout des composants de cration
      buttons = new Vector();
      
      // Les boutons d'outils
      for (i = 0; i < interf.getToolsCount(); i++) {
         button = new JSpacedButton (interf.getToolName(i), 2, 2, 2, 2);
         button.setToolTipText ("Outil: "+interf.getToolName(i));
         buttons.addElement (button);
         btPane.add (button);
      }

      // Label pour afficher l'outil en cours
      toolLabel = new JLabel ("["+interf.getCurToolName()+"]");
      toolLabel.setBorder ((Border)BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(2,8,2,8), BorderFactory.createLineBorder (Color.green)));
      toolLabel.setHorizontalAlignment (JLabel.CENTER);
      btPane.add (toolLabel);      
       
      // Capture des clics sur les boutons
      CreationButtonsListener cbl = new CreationButtonsListener();
      for (i=0; i < buttons.size(); i++)
        ((JButton)buttons.elementAt(i)).addActionListener (cbl);
      
      /* selPane: PANNEAU DE GESTION DES SELECTIONS                            */
      
      
      // Cration d'un sous-panneau pour les slctions
      JPanel selPane = new JPanel ();
      
      // Dfinition du gestionnaire de disposition pour le panneau des slctions
      selPane.setLayout (new FlowLayout());
      // Cration d'une bordure avec texte pour le panneau des slctions
      selPane.setBorder (BorderFactory.createTitledBorder (BorderFactory.createEtchedBorder (), "Slction"));
       
      // ComboBox contenant la liste des formes cres
      cbxSymbs = new JComboBox();
      cbxSymbs.setPreferredSize (new Dimension (130, 27));
      // Capture des clics sur le JComboBox
      cbxSymbs.addActionListener (new CreationCbxSymbsListener());
        
      // Ajout du combo box de slction au panneau de slction
      selPane.add (cbxSymbs);
      
      // Bouton de suppression
      btSuppr = new JButton ("Supprimer");
      
      // Capture des clics sur le bouton de suppression
      btSuppr.addActionListener (new SelectionButtonsListener ());
      
      // Ajout du bouton de suppression au panneau de slction
      selPane.add (btSuppr);
      
      
      /* opPane: PANNEAU DES OPTIONS DES FORMES/OUTILS                         */
          
       
      // Force l'affichage des panneaux en bordure
      setLayout (new BorderLayout());
      
      /* DISPOSITION DES PANNEAUX SUR LE PANNEAU DE CREATION                   */
           
       
      // Ajout des prcedents panneaux  ce panneau (la classe que l'on dfinit)
      btPane.setMinimumSize (new Dimension (150, 250) );
      btPane.setPreferredSize (new Dimension (150, 250) );
      selPane.setMinimumSize (new Dimension (150, 120) );
      selPane.setPreferredSize (new Dimension (150, 120) );
      add (btPane, BorderLayout.NORTH);
      add (selPane, BorderLayout.CENTER);
    } // end PanelCreation        

/**
 * 
 */
   PanelCreation(IHM interf) {        
      this (interf, 0, 1);
   } // end PanelCreation


/**
 * 
 */
   public void setSelection (Symbole sel) {
      cbxSymbs.setSelectedItem (sel);
   } // end setSelection

   public void addSymbole (Symbole s) {
      cbxSymbs.addItem(s);
   }
   
   public void removeSymbole (Symbole s) {
      cbxSymbs.setSelectedIndex (0);
      cbxSymbs.removeItem (s);
   }
   
   public void removeAllSymbs () {
      cbxSymbs.removeAllItems();
   }
   
   public void setToolLabel (int butIndex) {
      if ((butIndex >= 0) && (butIndex < buttons.size())) {
         toolLabel.setText( "[" + ((JSpacedButton)buttons.elementAt (butIndex)).getText() + "]" );
      }
   }
/**
 * 
 */
   private void setSelTool(int tool) {
      interf.creatChangeTool (tool);
    } // end setSelTool

/**
 * Listener pour les clics sur les boutons de cration
 */
class CreationButtonsListener implements ActionListener {

/**
 * 
 */
    public void actionPerformed(ActionEvent e) {        
        int i = buttons.indexOf(e.getSource());
        if (i >= 0)
          setSelTool(i);
    } // end actionPerformed        

} // end PanelCreation.CreationButtonsListener

/**
 * Listener pour les clics sur les boutons de slction
 */
class SelectionButtonsListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btSuppr) {
            interf.removeSelected();
        }
    } // end actionPerformed

} // end PanelCreation.SelectionButtonsListener

/**
 * Listener pour les clics sur cbxSymbs
 */
class CreationCbxSymbsListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {        
        interf.setSelection ( (Symbole)cbxSymbs.getSelectedItem() );
    } // end actionPerformed

} // end PanelCreation.CreationCbxSymbsListener

 } // end PanelCreation
