package reseau;

import java.util.Iterator;
import java.util.Vector;

public class MibModule {
	private		MibModuleLibrary	lib;
	protected 	String				name;
	protected	MibTree				tree;
	private		MibParser			parser;
	private		boolean				endOfFile;
	
	private		Vector<CustomType>	types;
	
	public MibModule (MibModuleLibrary m, MibParser p) {
		lib			= m;
		parser		= p;
		types		= new Vector<CustomType>();
		tree		= new MibTree();
		endOfFile	= false;
		//tree = t;
	}
	
	public String getName() {
		return name;
	}
	
	public void parse() throws MibParseErrorException {
		String lu, nextName;
		
		name = parser.nextItem();
		//System.out.println("Vla le module " + name);
		
		lu = parser.nextItem();
		
		if (lu.compareTo("DEFINITIONS")!=0)
			throw new MibParseErrorException();
		
		if (parser.nextItem().compareTo("::=")!=0)
			throw new MibParseErrorException();
		
		if (parser.nextItem().compareTo("BEGIN")!=0)
			throw new MibParseErrorException();

		parseImports();
		parseModuleId();
		
		do {
			lu = parser.nextItem();
			if (lu.compareTo("END") == 0) {
				endOfFile = true;
			} else {
				nextName = lu;
				lu = parser.nextItem();
				if (lu.compareTo("OBJECT") == 0) {
					parseNode(nextName);
				} else if (lu.compareTo("OBJECT-TYPE") == 0) {
					parseLeaf(nextName);
				} else if (lu.compareTo("::=") == 0) {
					parseType(nextName);
				} else {
					
					do {
						lu = parser.nextItem();
					} while (lu.compareTo("::=") != 0);
					do {
						lu = parser.nextItem();
					} while (lu.compareTo("}") != 0);
				}
			}
		} while (!endOfFile);
		
		tree.addToParent();
	}
	
	private void parseType (String nodeName) throws MibParseErrorException {
		String	lu, n, t;
		/*
		do {
			lu = parser.nextItem();
		} while (lu.compareTo("}") != 0);
		*/
		lu = parser.nextItem();
		if (lu.compareTo("SEQUENCE")==0) {
			CustomType	c = new CustomType(nodeName, CustomType.SEQUENCE);
			lu = parser.nextItem();
			if (lu.compareTo("{")!=0)
				throw new MibParseErrorException("parseType:"+nodeName);
			do {
				lu = parser.nextItem();
				if (lu.compareTo("}") != 0) {
					n = lu;
					t = parser.nextItem();
					if (t.compareTo("OBJECT")==0) {
						t += " " + parser.nextItem();
					}
					c.addSubType(n, t);
				}
			} while (lu.compareTo("}") != 0);
			addCustomType(c);
		} else if (lu.compareTo("TEXTUAL-CONVENTION")==0) {
			lu = parser.nextItem();
			if (lu.compareTo("DISPLAY-HINT") == 0) {
				lu = parser.nextItem();
				lu = parser.nextItem();
			}
			if (lu.compareTo("STATUS") != 0) {
				throw new MibParseErrorException("parseType:"+nodeName);
			}
			lu = parser.nextItem();
			lu = parser.nextItem();
			if (lu.compareTo("DESCRIPTION") != 0) {
				throw new MibParseErrorException("parseType:"+nodeName);
			}
			lu = parser.nextItem();
			lu = parser.nextItem();
			if (lu.compareTo("SYNTAX") != 0) {
				throw new MibParseErrorException("parseType:"+nodeName);
			}
			lu = parser.nextItem();
			parser.endOfLine();
			CustomType c = null;
			if (lu.compareTo("INTEGER") == 0) {
				c = new CustomType(lu, CustomType.INTEGER, 4);
			} else if (lu.compareTo("OBJECT") == 0) {
				c = new CustomType(lu, CustomType.OBJECT);
			} else if (lu.compareTo("OCTET") == 0) {
				c = new CustomType(lu, CustomType.OCTETSTRING);
			} else {
				CustomType c2 = getCustomType(lu);
				c = new CustomType(lu, c2.getType());
			}
			addCustomType(c);
		}
		
	}
	
	private void parseNode (String nodeName) throws MibParseErrorException {
		String	lu, parentName, num;
		int		n;
		
		lu = parser.nextItem();
		if (lu.compareTo("IDENTIFIER") != 0) {
			throw new MibParseErrorException("parseNode:"+nodeName);
		}
		lu = parser.nextItem();
		if (lu.compareTo("::=") != 0) {
			throw new MibParseErrorException("parseNode:"+nodeName);
		}
		lu = parser.nextItem();
		if (lu.compareTo("{") != 0) {
			throw new MibParseErrorException("parseNode:"+nodeName);
		}
		parentName = parser.nextItem();
		num = parser.nextItem();
		n = Integer.parseInt(num);
		if (n <= 0) {
			throw new MibParseErrorException("parseNode:"+nodeName);
		}
		lu = parser.nextItem();
		if (lu.compareTo("}") != 0) {
			throw new MibParseErrorException("parseNode:"+nodeName);
		}
		if (!tree.insertNode(parentName, n, nodeName, this))
			throw new MibParseErrorException("parseNode:"+nodeName);
	}
	
	private void parseLeaf(String nodeName) throws MibParseErrorException {
		String	lu, type, access, status, descr, parent, num;
		int		n;
	
		lu = parser.nextItem();
		if (lu.compareTo("SYNTAX") != 0) {
			throw new MibParseErrorException("parseLeaf1:"+nodeName);
		}
		type = parser.endOfLine();
		lu = parser.nextItem();
		if (lu.compareTo("MAX-ACCESS") != 0) {
			//throw new MibParseErrorException();
			do {
				lu = parser.nextItem();
			} while (lu.compareTo("MAX-ACCESS") != 0);
		}
		access = parser.nextItem();
		lu = parser.nextItem();
		if (lu.compareTo("STATUS") != 0) {
			throw new MibParseErrorException("parseLeaf2:"+nodeName);
		}
		status = parser.nextItem();
		lu = parser.nextItem();
		if (lu.compareTo("DESCRIPTION") != 0) {
			throw new MibParseErrorException("parseLeaf3:"+nodeName);
		}
		descr = parser.nextItem();
		lu = parser.nextItem();
		
		if (lu.compareTo("INDEX") == 0) {
			parser.endOfLine();
			lu = parser.nextItem();
		}
		
		if (lu.compareTo("AUGMENTS") == 0) {
			parser.endOfLine();
			lu = parser.nextItem();
		}
		
		if (lu.compareTo("::=") != 0) {
			//throw new MibParseErrorException("lecture d'une feuille " + nodeName);
			do {
				lu = parser.nextItem();
			} while (lu.compareTo("::=") != 0);
		}
		lu = parser.nextItem();
		if (lu.compareTo("{") != 0) {
			throw new MibParseErrorException("parseLeaf4:"+nodeName);
		}
		parent = parser.nextItem();
		num = parser.nextItem();
		lu = parser.nextItem();
		n = Integer.parseInt(num);
		if (n <= 0) {
			throw new MibParseErrorException("parseLeaf5:"+nodeName);
		}
		if (lu.compareTo("}") != 0) {
			throw new MibParseErrorException();
		}

		if (type.length() > 10 && type.substring(0, 11).compareTo("SEQUENCE OF")==0) {
			if (!tree.insertNode(parent, n, nodeName, this))
				throw new MibParseErrorException("parseLeaf6:"+nodeName);
		} else {

			if (!tree.insertLeaf(parent, n, nodeName, this, access, status, descr, type))
				throw new MibParseErrorException("parseLeaf7:"+nodeName);
		}
	}
	
	private void parseImports() throws MibParseErrorException {
		Vector<String> v = new Vector<String>();
		String lu;
		
		if (parser.nextItem().compareTo("IMPORTS")==0) {
			do {
				do {
					lu = parser.nextItem();
					v.add(lu);
				} while (parser.endChar() == ',');
				
				lu = parser.nextItem();
				if (lu.compareTo("FROM")==0) {
					makeImports(parser.nextItem(), v);
				} else throw new MibParseErrorException("erreur de parseImports");
				
				v.clear();
			} while (parser.endChar() != ';');
		} else throw new MibParseErrorException("erreur de parseImports");
	}
	
	private void parseModuleId() throws MibParseErrorException {
		String	lu, nodeName, parentName, num;
		int		n;
		
		nodeName = parser.nextItem();
		lu = parser.nextItem();
		if (lu.compareTo("MODULE-IDENTITY")!=0) {
			throw new MibParseErrorException();
		} else {
			while (lu.compareTo("::=")!=0) {
				lu = parser.nextItem();
			}
			if (parser.nextItem().compareTo("{") != 0) {
				throw new MibParseErrorException("erreur de parseModuleId");
			}
			parentName = parser.nextItem();
			//System.out.println("parentName : " + parentName);
			num = parser.nextItem();
			//System.out.println("num : " + num);
			n = Integer.parseInt(num);
			if (n <= 0) {
				throw new MibParseErrorException("erreur de parseModuleId");
			}
			if (parser.nextItem().compareTo("}") != 0) {
				throw new MibParseErrorException("erreur de parseModuleId");
			}
			if (!tree.insertRootNode(parentName, n, nodeName, this))
				throw new MibParseErrorException("erreur de parseModuleId");
		}
	}
	
	private void makeImports(String module, Vector<String> elements) throws MibParseErrorException {
		
		//System.out.println("Import depuis le module " + module);
		/*
		for (int i=0; i<elements.size(); i++)
			System.out.println("- " + elements.get(i));*/
		
		MibModule			m;
		Iterator<String>	i;
		String				s;
		MibNode				n;
		CustomType			c;
		
		m = lib.getModule(module);
		
		if (m == null) {
			throw new MibParseErrorException("MIB manquante : " + module);
		}
		
		i = elements.iterator();
		while (i.hasNext()) {
			s = i.next();
			if ( (n = (m.getTree()).findNode(s)) != null ) {
				tree.addNode(n);
				//System.out.println(s + " ajout comme node");
			} else {
				if ( (c = m.getCustomType(s) ) != null ) {
					//System.out.println(s + " ajout comme type");
					addCustomType(c);
				} else {
					//System.out.println("Erreur d'import pour " + s);
				}
			}
		}
	}
	
	protected void addCustomType(CustomType c) {
		types.add(c);
	}
	
	protected CustomType getCustomType(String s) {
		Iterator<CustomType>	i = types.iterator();
		CustomType				c;
		
		//System.out.println("Recherche de " + s + " dans les types du module " + name);
		
		while (i.hasNext()) {
			c = (CustomType)i.next();
			//System.out.print(c.getName() + "...");
			if (c.getName().compareTo(s) == 0) {
				//System.out.println("ok");
				return c;
			}
			//System.out.println("no");
		}
		return null;
	}
	
	protected MibTree getTree() {
		return tree;
	}
	
	public void displayModule() {
		tree.displayTree();
	}
	
	public void display() {
		tree.display();
	}
}
