package reseau;

public abstract class Type {
	public static final byte INTEGER	= 0x02;
	public static final byte STRING		= 0x04;
	public static final byte NULL		= 0x05;
	public static final byte OBJECT		= 0x06;
	public static final byte SEQUENCE	= 0x30;
	public static final byte IPADDRESS	= 0x40;
	public static final byte COUNTER	= 0x41;
	public static final byte GAUGE		= 0x42;
	public static final byte TIMETICKS	= 0x43;
	public static final byte OPAQUE		= 0x44;
	public static final byte COUNTER64	= 0x46;
	public static final byte GET		= (byte)0xa0;
	public static final byte GETNEXT	= (byte)0xa1;
	public static final byte RESPONSE	= (byte)0xa2;
	public static final byte SET		= (byte)0xa3;
	public static final byte TRAP		= (byte)0xa4;

	protected byte	type;
	protected int	length;
	protected byte	content[];
	
	public Type(byte t, int l) {
		type = t;
		length = l;
	}
	
	public final byte getType() {
		return type;
	}
	
	public final void setType(byte t) {
		type = t;
	}
	
	public final byte[] buildLength(int l) {
		byte	tab[];
	
		if (l < 128) {
			tab = new byte[1];
			tab[0] = (byte)l;
		} else {
			int p = l;
			
			int n = 1;
			while (p > 255) {
				n++;
				p = p/256;
			}
			
			tab = new byte[n+1];
			tab[0] = (byte)((byte)0x80 | (byte)n);
			int r = l;
			for (int i=n; i>0; i--) {
				tab[i] = (byte)(r%256);
				r = r/256;
			}
		}
		
		return tab;
	}
	
	public final byte[] getBytes() {
		//byte	tab[] = new byte[2 + length];
		
		byte 	length_tab[] = buildLength(length);
		byte	tab[] = new byte[1 + length_tab.length + length];
		
		tab[0] = type;
		for (int i=0; i<length_tab.length; i++) {
			tab[i+1] = length_tab[i];
		}
		
		for (int i=0; i<length; i++)
			tab[i+length_tab.length+1] = content[i];
			
		return tab;
	}
	
	public abstract boolean fromString (String s);
	
	protected abstract void updateContent();
	
	public static Message messageFromBytes(byte tab[], int isos) {
		byte	type = tab[0];
		int		len=0, n=0;
		byte	newtab[];
		Message	ret = null;
		
		if (tab[1] >= 0) {
			n = 0;
			len = tab[1];
		} else {
			n = tab[1] + 128;
			int p = 1;
			for (int i=n; i>0; i--) {
				if (tab[i+1] < 0) {
					len += (tab[i+1] + 256) * p;
				} else {
					len += tab[i+1] * p;
				}
				p *= 256;
			}
		}
		
		newtab = new byte[len];
		for (int i=0; i<len; i++)
			newtab[i] = tab[i+n+2];
		
		switch (type) {
		case Type.SEQUENCE:
			ret = new Message(len, newtab, isos);
			break;
		default:
		}
		
		return ret;
	}
	
	/*
	public static Type typeFromBytes(byte tab[]) {
		byte	type = tab[0];
		int		len=0, n=0;
		byte	newtab[];
		Type	ret = new TypeNull();
		
		if (tab[1] >= 0) {
			n = 0;
			len = tab[1];
		} else {
			n = tab[1] + 128;
			int p = 1;
			for (int i=n; i>=0; i--) {
				if (tab[i+1] < 0) {
					len += (tab[i+1] + 256) * p;
				} else {
					len += tab[i+1] * p;
				}
				p *= 256;
			}
		}
		
		newtab = new byte[len];
		for (int i=0; i<len; i++)
			newtab[i] = tab[i+n+2];
		
		switch (type) {
		case Type.INTEGER:
		case Type.COUNTER:
		case Type.GAUGE:
		case Type.TIMETICKS:
		case Type.COUNTER64:
			ret = new TypeInteger(type, len, newtab);
			break;
		case Type.IPADDRESS:
			ret = new TypeIpAddress(len, newtab);
			break;
		case Type.STRING:
			ret = new TypeString(len, newtab);
			break;
		case Type.NULL:
			ret = new TypeNull();
			break;
		case Type.OBJECT:
			ret = new TypeObject(len, newtab);
			break;
		case Type.SEQUENCE:
			ret = new TypeSequence(len, newtab);
			break;
		case Type.GET:
		case Type.GETNEXT:
		case Type.SET:
		case Type.RESPONSE:
			ret = new PDU(type, len, newtab);
			break;
		case Type.TRAP:
			ret = new TrapPDU(len, newtab);
			break;
		default:
		}
		
		return ret;
	}*/
	
	public static Type typeFromBytes(byte tab[], int isos) {
		byte	type = tab[0];
		int		len=0, n=0;
		byte	newtab[];
		Type	ret = new TypeNull();

		//System.out.println("tab[1] = " + tab[1]);
		if (tab[1] >= 0) {
			n = 0;
			len = tab[1];
		} else {
			n = tab[1] + 128;
			//System.out.println("n=" + n);
			int p = 1;
			for (int i=n; i>0; i--) {
				//System.out.println("tab["+(i+1)+"]="+tab[i+1]);
				if (tab[i+1] < 0) {
					len += (tab[i+1] + 256) * p;
				} else {
					len += tab[i+1] * p;
				}
				p *= 256;
			}
		}
		
		//System.out.println("len="+len);
		//System.out.println("n="+n);
		//System.out.println("tab.length="+tab.length);
		
		newtab = new byte[len];
		for (int i=0; i<len; i++) {
			//System.out.println("newtab-type["+i+"]="+Integer.toHexString(tab[i+n+2]));
			newtab[i] = tab[i+n+2];
		}
		
		switch (type) {
		case Type.INTEGER:
		case Type.COUNTER:
		case Type.GAUGE:
		case Type.TIMETICKS:
		case Type.COUNTER64:
			ret = new TypeInteger(type, len, newtab);
			break;
		case Type.IPADDRESS:
			ret = new TypeIpAddress(len, newtab);
			break;
		case Type.STRING:
			if (isos==0)
				ret = new TypeOctetString(len, newtab);
			else
				ret = new TypeString(len, newtab);
			break;
		case Type.NULL:
			ret = new TypeNull();
			break;
		case Type.OBJECT:
			ret = new TypeObject(len, newtab);
			break;
		case Type.SEQUENCE:
			ret = new TypeSequence(len, newtab, isos-1);
			break;
		case Type.GET:
		case Type.GETNEXT:
		case Type.SET:
		case Type.RESPONSE:
			ret = new PDU(type, len, newtab, isos-1);
			break;
		case Type.TRAP:
			ret = new TrapPDU(len, newtab);
			break;
		default:
		}
		
		return ret;
	}
}
